/*
  pyAB_prj.c

*/
#include "simpletools.h"
#include "simpletext.h"
#include "fdserial.h"
#include "abdrive.h"
#include "ping.h"
#include "sonyremote.h"

volatile unsigned int IRpin = 8;

static volatile int xbCog;
static volatile int speedLeft = 0;
static volatile int speedRight = 0;
static volatile int roamcmd;
static volatile int speedLeftOld = 0;
static volatile int speedRightOld = 0;

serial *xbee;

// Function prototype
int IR_remote(int irPin);
int XB_remote(void);
void ab_nav(void);
void global_Roam(void);

int main()
{
  // Add startup code here.

  char inBuff[40];
/*                     Rx  Tx mode BAUD */
  xbee = fdserial_open(7, 6, 0, 9600);
  pause(500);

  int roamcmdOld = roamcmd;
  int speedLold2 = speedLeft;
  int speedRold2 = speedRight;

  int key = -1, xbcmd = -1;

  while(1)
  {
    // Add main loop code here.
    key = IR_remote(IRpin);
    xbcmd = XB_remote();
    if(xbcmd == 1) global_Roam();
    ab_nav();    
  }  
}

///////////////////////////////////////////////////////////////////////////////
/* IR remote control */
int IR_remote(int irPin)
{
  int key;
  ir_tLimit(50);

  key = ir_key(irPin);
  if(key == PWR)
  {
    speedLeft = 0;
    speedRight = 0;
  }
  if(key == CH_UP)  // CH_UP -> Forward
  {
    speedLeft = 32;
    speedRight = 32;
  }
  if(key == CH_DN)  // CH_DN -> Backward
  {
    speedLeft = -32;
    speedRight = -32;
  }
  if(key == VOL_UP)  // VOL_UP -> Right
  {
    speedLeft = 16;
    speedRight = 0;
  }
  if(key == VOL_DN)  // VOL_DN -> Left
  {
    speedLeft = 0;
    speedRight = 16;
  }
  if(key == MUTE)
  {
    //fclose(fp);
    //high(26);
    //while(1);
  }
  if(key == ENTER)
  {
    //fp = fopen("log.txt", "w");
  }

  if(key != -1)
  {
    // set up string for value
    //char val[] = {' ', ' ', ' ', '\r', '\n'};
    // key to string over spaces in val
    //itoa(key, val, 10);    
    // display in terminal to verify
    //putLine(val);  
    // store string to log.txt
    //fwrite(val, 1, 6, fp); 
  }

  return key; 
}

///////////////////////////////////////////////////////////////////////////////
/* XB remote control */
int XB_remote()
{
  char inBuff[40];
  char inDuff[5];
  int retval = 0;

if(fdserial_rxReady(xbee))
  {    
    readStr(xbee, inBuff, 40);
    //writeStr(xbee,"> ");
    if(!strcmp(inBuff,"onled"))
    {
      high(26);
      //writeStr(xbee,"logtime");
    }
    else if(!strcmp(inBuff,"offled"))
    {
      low(26);
      //writeStr(xbee,"logtime");
    }
    else if(!strcmp(inBuff,"gostop"))
    {
      speedLeft = 0;
      speedRight = 0;
    }
    else if(!strcmp(inBuff,"gofore"))
    {
      speedLeft = 32;
      speedRight = 32;
    }
    else if(!strcmp(inBuff,"goback"))
    {
      speedLeft = -32;
      speedRight = -32;
    }
    else if(!strcmp(inBuff,"goright"))
    {
      speedLeft = 16;
      speedRight = 0;
    }
    else if(!strcmp(inBuff,"goleft"))
    {
      speedLeft = 0;
      speedRight = 16;
    }
    else if(!strcmp(inBuff,"startroam"))
    {
      retval = 1;
    }
    else if(!strcmp(inBuff,"stoproam"))
    {
      retval = 2;
      speedLeft = 0;
      speedRight = 0;
    }
    else if(!strcmp(inBuff,"runprog"))
    {
      while(1) // Start of interpreter
      {
        readStr(xbee,inBuff,40);
        if(!strcmp(inBuff,"::")) break; // End of script
        if(!strcmp(inBuff,"onled26"))
        {
          high(26);
          writeStr(xbee,"#");  // Send next command.
        }
        if(!strcmp(inBuff,"offled26"))
        {
          low(26);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"onled27"))
        {
          high(27);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"offled27"))
        {
          low(27);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait1000"))
        {
          pause(1000);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait500"))
        {
          pause(500);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait250"))
        {
          pause(250);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait100"))
        {
          pause(100);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait50"))
        {
          pause(50);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"wait10"))
        {
          pause(10);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"gostop"))
        {
          speedLeft = 0;
          speedRight = 0;
          writeStr(xbee,"#");          
        }
        if(!strcmp(inBuff,"gofore"))
        {
          //print("%s\n",inBuff); // debug check value = gofore
          writeStr(xbee,"#"); // signal to send next value
          pause(100);          
          readStr(xbee,inDuff,10); // incoming value is "4\n"
          pause(100);
          int d = atoi(inDuff);  //        
          drive_goto(d*7.8,d*7.8);  // approximate conversion to one inch
          pause(100);
          writeStr(xbee,"#"); // send control value
        }
        if(!strcmp(inBuff,"goback"))
        {
          writeStr(xbee,"#"); // signal to send next value
          pause(100);          
          readStr(xbee,inDuff,10); //
          pause(100);
          //print("%s\n",inDuff);
          int d = atoi(inDuff);  //         
          drive_goto(-d*7.8,-d*7.8);  // approximate conversion to one inch
          pause(100);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"goleft"))
        {
          writeStr(xbee,"#"); // signal to send next value
          pause(100);          
          readStr(xbee,inDuff,10); //
          pause(100);
          //print("%s\n",inDuff);
          int d = atoi(inDuff);  //          
          drive_goto(0,d*.57);  // approximate conversion to one degree
          pause(100);
          writeStr(xbee,"#");
        }
        if(!strcmp(inBuff,"gorite"))
        {
          writeStr(xbee,"#"); // signal to send next value
          pause(100);          
          readStr(xbee,inDuff,10); //
          pause(100);
          //print("%s\n",inDuff);
          int d = atoi(inDuff);  //          
          drive_goto(d*.57,0);  // approximate conversion to one degree
          pause(100);
          writeStr(xbee,"#");
        }
        pause(100);
      } // End of interpreter.
    }
    else
    {
      writeLine(xbee,"Invalid Command!");
      retval = -1;
    }
  }
  return retval;   
}

///////////////////////////////////////////////////////////////////////////////
void ab_nav(void)
{
  if(speedLeftOld != speedLeft || speedRightOld != speedRight)
  {
    drive_speed(speedLeft, speedRight);
    speedLeftOld = speedLeft;
    speedRightOld = speedRight;
  } 
  pause(20);
}

///////////////////////////////////////////////////////////////////////////////

void global_Roam(void)
{
  int turn;
  roamcmd = 0;
  while(1)
  {
    if(XB_remote() == 2)
      return;
    else
    {
      speedLeft = 32;
      speedRight = 32;
      ab_nav();
      pause(100);
      while(ping_cm(5) >= 20)  // Ping is p5
      {
        pause(5);
        if(XB_remote() == 2) return;
      }
      if(XB_remote() == 2) return;

      turn = rand() % 2;
      if(turn == 1)

      {
        speedLeft = 16;
        speedRight = -16;
      }
      else
      {
        speedLeft = -16;
        speedRight = 16;
      }
      ab_nav();
      while(ping_cm(5) < 20);  // Ping is p5
      {
        pause(5);
        if(XB_remote() == 2) return;
      }
    }
  } 
}
